import Client from "discord-request";
import { URL } from "url";
import Config from "./config";
import { API_URL } from "../definitions";
// TODO: Test, Type, Document
export async function getGlobalApplicationCommands(options = {}) {
    const { applicationID = Config.getApplicationID(), headers = Config.getHeaders(), } = options;
    const url = new URL(`${API_URL}/applications/${applicationID}/commands`);
    const response = await Client.get(url, {
        headers,
    }, {
        route: "[GET] /applications/{application.id}/commands",
        identifier: applicationID,
    });
    return response.json();
}
// TODO: Test, Type, Document
export async function postGlobalApplicationCommand(command, options = {}) {
    const { applicationID = Config.getApplicationID(), headers = Config.getHeaders(), } = options;
    const url = new URL(`${API_URL}/applications/${applicationID}/commands`);
    const response = await Client.post(url, {
        headers,
        body: JSON.stringify(command),
    }, {
        route: "[POST] /applications/{application.id}/commands",
        identifier: applicationID,
    });
    return response.json();
}
// TODO: Test, Type, Document
export async function putGlobalApplicationCommands(commands, options = {}) {
    const { applicationID = Config.getApplicationID(), headers = Config.getHeaders(), } = options;
    const url = new URL(`${API_URL}/applications/${applicationID}/commands`);
    const response = await Client.put(url, {
        headers,
        body: JSON.stringify(commands),
    }, {
        route: "[PUT] /applications/{application.id}/commands",
        identifier: applicationID,
    });
    return response.json();
}
// TODO: Test, Type, Document
export async function patchGlobalApplicationCommand(command, options = {}) {
    const { applicationID = Config.getApplicationID(), headers = Config.getHeaders(), } = options;
    const url = new URL(`${API_URL}/applications/${applicationID}/commands/${command.id}`);
    const response = await Client.patch(url, {
        headers,
        body: JSON.stringify(command),
    }, {
        route: "[PATCH] /applications/{application.id}/commands",
        identifier: applicationID,
    });
    return response.json();
}
// TODO: Test, Type, Document
export async function deleteGlobalApplicationCommand(commandID, options = {}) {
    const { applicationID = Config.getApplicationID(), headers = Config.getHeaders(), } = options;
    const url = new URL(`${API_URL}/applications/${applicationID}/commands/${commandID}`);
    const response = await Client.delete(url, {
        headers,
    }, {
        route: "[DELETE] /applications/{application.id}/commands/{command.id}",
        identifier: applicationID,
    });
    return response.ok;
}
// TODO: Test, Type, Document
export async function getGuildApplicationCommands(guildID, options = {}) {
    const { applicationID = Config.getApplicationID(), headers = Config.getHeaders(), } = options;
    const url = new URL(`${API_URL}/applications/${applicationID}/guilds/${guildID}/commands`);
    const response = await Client.get(url, {
        headers,
    }, {
        route: "[GET] /applications/{application.id}/guilds/{guild.id}/commands",
        identifier: guildID,
    });
    return response.json();
}
// TODO: Test, Type, Document
export async function postGuildApplicationCommand(guildID, command, options = {}) {
    const { applicationID = Config.getApplicationID(), headers = Config.getHeaders(), } = options;
    const url = new URL(`${API_URL}/applications/${applicationID}/guilds/${guildID}/commands`);
    const response = await Client.post(url, {
        headers,
        body: JSON.stringify(command),
    }, {
        route: "[POST] /applications/{application.id}/guilds/{guild.id}/commands",
        identifier: guildID,
    });
    return response.json();
}
// TODO: Test, Type, Document
export async function putGuildApplicationCommands(guildID, commands, options = {}) {
    const { applicationID = Config.getApplicationID(), headers = Config.getHeaders(), } = options;
    const url = new URL(`${API_URL}/applications/${applicationID}/guilds/${guildID}/commands`);
    const response = await Client.put(url, {
        headers,
        body: JSON.stringify(commands),
    }, {
        route: "[PUT] /applications/{application.id}/guilds/{guild.id}/commands",
        identifier: guildID,
    });
    return response.json();
}
// TODO: Test, Type, Document
export async function patchGuildApplicationCommand(guildID, command, options = {}) {
    const { applicationID = Config.getApplicationID(), headers = Config.getHeaders(), } = options;
    const url = new URL(`${API_URL}/applications/${applicationID}/guilds/${guildID}/commands/${command.id}`);
    const response = await Client.patch(url, {
        headers,
        body: JSON.stringify(command),
    }, {
        route: "[PATCH] /applications/{application.id}/guilds/{guild.id}/commands",
        identifier: guildID,
    });
    return response.json();
}
// TODO: Test, Type, Document
export async function deleteGuildApplicationCommand(guildID, commandID, options = {}) {
    const { applicationID = Config.getApplicationID(), headers = Config.getHeaders(), } = options;
    const url = new URL(`${API_URL}/applications/${applicationID}/guilds/${guildID}/commands/${commandID}`);
    const response = await Client.delete(url, {
        headers,
    }, {
        route: "[DELETE] /applications/{application.id}/guilds/{guild.id}/commands/{command.id}",
        identifier: guildID,
    });
    return response.ok;
}
//# sourceMappingURL=commands.js.map