/// <reference types="node" />
/// <reference types="node/http" />
/// <reference types="got/dist/source/core/utils/timed-out" />
import type { FastifyRequest, FastifyReply } from "fastify";
import SlashCommand from "./commands/slash-command";
import ContextMenu from "./commands/context-menu";
import { Interaction as InteractionDefinition, ApplicationCommandType } from "./definitions";
import { InteractionKitCommand, SerializableComponent } from "./interfaces";
import ApplicationCommandInteraction from "./interactions/application-commands/application-command-interaction";
import { ExecutableComponent } from "./components";
declare type ApplicationArgs = {
    applicationID: string;
    publicKey: string;
    token: string;
    port?: number;
};
export interface CommandMap extends Map<ApplicationCommandType, Map<string, SlashCommand | ContextMenu<ApplicationCommandType.MESSAGE> | ContextMenu<ApplicationCommandType.USER>>> {
    get(key: ApplicationCommandType.CHAT_INPUT): Map<string, SlashCommand>;
    get(key: ApplicationCommandType.MESSAGE): Map<string, ContextMenu<ApplicationCommandType.MESSAGE>>;
    get(key: ApplicationCommandType.USER): Map<string, ContextMenu<ApplicationCommandType.USER>>;
    get(key: ApplicationCommandType): Map<string, InteractionKitCommand<ApplicationCommandInteraction>>;
}
export default class Application {
    #private;
    constructor({ applicationID, publicKey, token, port }: ApplicationArgs);
    get id(): `${bigint}`;
    get commands(): (SlashCommand | ContextMenu<ApplicationCommandType.MESSAGE> | ContextMenu<ApplicationCommandType.USER>)[];
    addCommand(command: InteractionKitCommand<ApplicationCommandInteraction>): this;
    addCommands(...commands: Array<InteractionKitCommand<ApplicationCommandInteraction>>): this;
    addComponent(component: SerializableComponent): this;
    getComponent(customID: string): ExecutableComponent | undefined;
    getCommand(type: ApplicationCommandType, name: string): InteractionKitCommand<ApplicationCommandInteraction> | undefined;
    loadApplicationCommandDirectory(directoryPath: string): this;
    loadMessageComponentDirectory(directoryPath: string): this;
    handler(request: FastifyRequest<{
        Body: InteractionDefinition;
    }>, response: FastifyReply): void;
    startServer(): Promise<import("fastify").FastifyInstance<import("http").Server, import("http").IncomingMessage, import("http").ServerResponse, import("fastify").FastifyLoggerInstance>>;
}
export {};
