var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Application_applicationID, _Application_publicKey, _Application_token, _Application_commands, _Application_components, _Application_port;
import fs from "node:fs";
import path from "node:path";
import url from "node:url";
import Config from "./api/config";
import { ApplicationCommandType, } from "./definitions";
import * as Interaction from "./interactions";
import startInteractionKitServer from "./server";
import { isExecutableComponent } from "./components";
export default class Application {
    constructor({ applicationID, publicKey, token, port }) {
        _Application_applicationID.set(this, void 0);
        _Application_publicKey.set(this, void 0);
        _Application_token.set(this, void 0);
        _Application_commands.set(this, void 0);
        _Application_components.set(this, new Map());
        _Application_port.set(this, void 0);
        if (!applicationID) {
            throw new Error("Please provide an Application ID. You can find this value <here>");
        }
        if (!publicKey) {
            throw new Error("Please provide a Public Key. You can find this value <here>");
        }
        if (!token) {
            throw new Error("Please provide a Token. You can find this value <here>");
        }
        __classPrivateFieldSet(this, _Application_applicationID, applicationID, "f");
        __classPrivateFieldSet(this, _Application_publicKey, publicKey, "f");
        __classPrivateFieldSet(this, _Application_token, token, "f");
        __classPrivateFieldSet(this, _Application_port, port ?? 3000, "f");
        // Set up internal data structures
        __classPrivateFieldSet(this, _Application_commands, new Map([
            [ApplicationCommandType.CHAT_INPUT, new Map()],
            [ApplicationCommandType.MESSAGE, new Map()],
            [ApplicationCommandType.USER, new Map()],
        ]), "f");
        // Configure API Defaults
        Config.setToken(__classPrivateFieldGet(this, _Application_token, "f"));
        Config.setApplicationID(__classPrivateFieldGet(this, _Application_applicationID, "f"));
    }
    get id() {
        return __classPrivateFieldGet(this, _Application_applicationID, "f");
    }
    get commands() {
        return Array.from(__classPrivateFieldGet(this, _Application_commands, "f").values())
            .map((map) => Array.from(map.values()))
            .flat();
    }
    addCommand(command) {
        if (__classPrivateFieldGet(this, _Application_commands, "f").get(command.type)?.has(command.name.toLowerCase())) {
            throw new Error(`Error registering ${command.name.toLowerCase()}: Duplicate names are not allowed`);
        }
        console.log(`Registering the ${command.name.toLowerCase()} command`);
        __classPrivateFieldGet(this, _Application_commands, "f").get(command.type)?.set(command.name.toLowerCase(), command);
        return this;
    }
    addCommands(...commands) {
        commands.forEach((command) => this.addCommand(command));
        return this;
    }
    addComponent(component) {
        if (isExecutableComponent(component) &&
            component.id != null &&
            !__classPrivateFieldGet(this, _Application_components, "f").has(component.id)) {
            __classPrivateFieldGet(this, _Application_components, "f").set(component.id, component);
        }
        return this;
    }
    getComponent(customID) {
        return __classPrivateFieldGet(this, _Application_components, "f").get(customID);
    }
    getCommand(type, name) {
        return __classPrivateFieldGet(this, _Application_commands, "f").get(type).get(name);
    }
    loadApplicationCommandDirectory(directoryPath) {
        const directory = path.join(path.dirname(url.fileURLToPath(import.meta.url)), directoryPath);
        fs.readdir(directory, async (err, files) => {
            if (err) {
                throw err;
            }
            for (const file of files) {
                if (file.endsWith(".js")) {
                    const command = await import(path.join(directory, file));
                    this.addCommand(command.default);
                }
            }
        });
        return this;
    }
    loadMessageComponentDirectory(directoryPath) {
        const directory = path.join(path.dirname(url.fileURLToPath(import.meta.url)), directoryPath);
        fs.readdir(directory, async (err, files) => {
            if (err) {
                throw err;
            }
            for (const file of files) {
                if (file.endsWith(".js")) {
                    const component = await import(path.join(directory, file));
                    this.addComponent(component.default);
                }
            }
        });
        return this;
    }
    handler(request, response) {
        console.log("REQUEST");
        try {
            Interaction.handler(this, request, response);
        }
        catch (exception) {
            console.log(exception);
            void response.status(400).send({
                error: "Unknown Type",
            });
            throw exception;
        }
    }
    async startServer() {
        return startInteractionKitServer((...args) => {
            this.handler(...args);
        }, __classPrivateFieldGet(this, _Application_publicKey, "f"), __classPrivateFieldGet(this, _Application_port, "f"));
    }
}
_Application_applicationID = new WeakMap(), _Application_publicKey = new WeakMap(), _Application_token = new WeakMap(), _Application_commands = new WeakMap(), _Application_components = new WeakMap(), _Application_port = new WeakMap();
//# sourceMappingURL=application.js.map