import type { Executable, SerializableComponent } from "../interfaces";
import Application from "../application";
import { ButtonStyle, Component, ComponentType } from "../definitions";
import ButtonInteraction from "../interactions/message-components/button-interaction";
declare type ButtonBaseArgs = Omit<Component, "type" | "url" | "custom_id" | "components" | "options" | "placeholder" | "min_values" | "max_values">;
declare type ButtonArgs = {
    handler: (event: ButtonInteraction, application: Application) => unknown;
    customID: Component["custom_id"];
    style: Exclude<ButtonStyle, ButtonStyle.LINK>;
} & ButtonBaseArgs;
declare type ButtonLinkArgs = Omit<Component, "type" | "custom_id" | "style" | "components">;
declare abstract class ButtonBase implements SerializableComponent {
    #private;
    constructor(options: ButtonBaseArgs);
    get id(): SerializableComponent["id"];
    get type(): ComponentType;
    setLabel(label: ButtonArgs["label"]): this;
    setEmoji(emoji: ButtonArgs["emoji"]): this;
    setDisabled(disabled: ButtonArgs["disabled"]): this;
    serialize(): Component;
    protected setStyle(style: ButtonStyle): this;
}
export declare class ButtonLink extends ButtonBase {
    #private;
    constructor(options: ButtonLinkArgs);
    setURL(url: ButtonLinkArgs["url"]): this;
    serialize(): Component;
}
export declare class Button extends ButtonBase implements SerializableComponent, Executable<ButtonInteraction> {
    #private;
    handler: ButtonArgs["handler"];
    constructor(options: ButtonArgs);
    get id(): string | undefined;
    setCustomID(customID: ButtonArgs["customID"]): this;
    setStyle(style: Exclude<ButtonArgs["style"], ButtonStyle.LINK>): this;
    setHandler(fn: ButtonArgs["handler"]): this;
    serialize(): Component;
}
export {};
