export class Choices {
    constructor(choices) {
        this._choices = new Map();
        Object.entries(choices).forEach(([key, value]) => {
            this._choices.set(key, value);
        });
    }
    static create(choices) {
        return Object.freeze(new this(choices));
    }
    serialize() {
        return Array.from(this._choices.values());
    }
}
export class SelectOptionList extends Choices {
    constructor(choices) {
        const values = {};
        Object.entries(choices).forEach(([key, value]) => {
            if (typeof value === "string") {
                values[key] = { label: value, value };
            }
            else if (value instanceof Object) {
                values[key] = value;
            }
            else {
                throw new Error("Unknown Input Format");
            }
        });
        super(values);
        Object.entries(values).forEach(([key, value]) => {
            Object.defineProperty(this, key, {
                writable: false,
                value,
            });
        });
    }
}
export class SlashChoiceList extends Choices {
    constructor(choices) {
        const values = {};
        Object.entries(choices).forEach(([key, value]) => {
            if (typeof value === "string") {
                values[key] = { name: value, value };
            }
            else if (typeof value === "number") {
                values[key] = { name: value.toString(), value };
            }
            else if (value instanceof Object) {
                values[key] = value;
            }
            else {
                throw new Error("Unknown Input Format");
            }
        });
        super(values);
        Object.entries(values).forEach(([key, value]) => {
            Object.defineProperty(this, key, {
                writable: false,
                value: value.value,
            });
        });
    }
}
//# sourceMappingURL=choices.js.map