import { Comparable, Serializable } from "../interfaces";
import { ApplicationCommandOption, ApplicationCommandOptionChoice, ApplicationCommandOptionType } from "../definitions";
import { SlashChoiceList } from "./choices";
declare type InputChoiceValue = ApplicationCommandOptionChoice["value"];
declare type InputArgs = {
    type: ApplicationCommandOptionType;
    name: string;
    description: string;
    required?: boolean;
    choices?: SlashChoiceList<InputChoiceValue>;
    options?: ApplicationCommandOption[];
};
export declare class Input implements Serializable, Comparable<ApplicationCommandOption> {
    readonly type: ApplicationCommandOptionType;
    readonly name: string;
    readonly description: string;
    readonly required: boolean;
    readonly options: ApplicationCommandOption[] | undefined;
    readonly choices: SlashChoiceList<string | number> | undefined;
    constructor({ type, name, description, choices, options, required, }: InputArgs);
    serialize(): ApplicationCommandOption;
    equals(schema: ApplicationCommandOption): boolean;
}
interface StringInputArgs extends Omit<InputArgs, "type" | "options"> {
    choices?: SlashChoiceList<string>;
}
export declare class StringInput extends Input {
    constructor(args: StringInputArgs);
}
interface IntegerInputArgs extends Omit<InputArgs, "type" | "options"> {
    choices?: SlashChoiceList<number>;
}
export declare class IntegerInput extends Input {
    constructor(args: IntegerInputArgs);
}
export declare class BooleanInput extends Input {
    constructor(args: Omit<InputArgs, "type" | "choices" | "options">);
}
export declare class UserInput extends Input {
    constructor(args: Omit<InputArgs, "type" | "choices" | "options">);
}
export declare class ChannelInput extends Input {
    constructor(args: Omit<InputArgs, "type" | "choices" | "options">);
}
export declare class RoleInput extends Input {
    constructor(args: Omit<InputArgs, "type" | "choices" | "options">);
}
export declare class MentionableInput extends Input {
    constructor(args: Omit<InputArgs, "type" | "choices" | "options">);
}
export {};
