import { ApplicationCommandOptionType, } from "../definitions";
export class Input {
    constructor({ type, name, description, choices, options, required = false, }) {
        this.type = type;
        this.name = name;
        this.description = description;
        this.required = required;
        this.choices = choices;
        this.options = options;
    }
    serialize() {
        const payload = {
            type: this.type,
            name: this.name,
            description: this.description,
            required: this.required,
        };
        if (this.choices != null) {
            payload.choices = this.choices.serialize();
        }
        // TODO: Handle Options (do we want to?)
        return payload;
    }
    equals(schema) {
        if (this.type !== schema.type ||
            this.name !== schema.name ||
            this.description !== schema.description ||
            this.required !== (schema.required ?? false)) {
            return false;
        }
        if ((this.choices?._choices?.size ?? 0) !== (schema.choices?.length ?? 0)) {
            return false;
        }
        const choiceMap = new Map();
        for (const commandOption of this.choices?._choices?.values() ?? []) {
            choiceMap.set(commandOption.name, commandOption.value);
        }
        const choiceEquality = schema.choices?.every((choice) => {
            if (!choiceMap.has(choice.name)) {
                return false;
            }
            return choiceMap.get(choice.name) === choice.value;
        }) ?? true;
        if (!choiceEquality) {
            return false;
        }
        // TODO: Nothing uses options yet, but eventually verify they're correct somehow
        // if ((this.options?.length ?? 0) !== (schema.options?.length ?? 0)) {
        // 	return false;
        // }
        return true;
    }
}
export class StringInput extends Input {
    constructor(args) {
        super({ type: ApplicationCommandOptionType.STRING, ...args });
    }
}
export class IntegerInput extends Input {
    constructor(args) {
        super({ type: ApplicationCommandOptionType.INTEGER, ...args });
    }
}
export class BooleanInput extends Input {
    constructor(args) {
        super({ type: ApplicationCommandOptionType.BOOLEAN, ...args });
    }
}
export class UserInput extends Input {
    constructor(args) {
        super({ type: ApplicationCommandOptionType.USER, ...args });
    }
}
export class ChannelInput extends Input {
    constructor(args) {
        super({ type: ApplicationCommandOptionType.CHANNEL, ...args });
    }
}
export class RoleInput extends Input {
    constructor(args) {
        super({ type: ApplicationCommandOptionType.ROLE, ...args });
    }
}
export class MentionableInput extends Input {
    constructor(args) {
        super({ type: ApplicationCommandOptionType.MENTIONABLE, ...args });
    }
}
//# sourceMappingURL=inputs.js.map