import { webcrypto } from "crypto";
import { TextEncoder } from "util";
const encoder = new TextEncoder();
export function hexToBinary(hex) {
    const buffer = new Uint8Array(Math.ceil(hex.length / 2));
    for (let i = 0; i < buffer.length; i++) {
        buffer[i] = parseInt(hex.substr(i * 2, 2), 16);
    }
    return buffer;
}
/* eslint-disable */
export function getPublicKey(publicKey) {
    // @ts-ignore ????
    return webcrypto.subtle.importKey("raw", hexToBinary(publicKey), {
        name: "NODE-ED25519",
        namedCurve: "NODE-ED25519",
        public: true,
    }, true, ["verify"]);
}
export async function validateRequest(request, publicKey) {
    const signature = hexToBinary(request.headers["x-signature-ed25519"]);
    const timestamp = request.headers["x-signature-timestamp"];
    const body = request.rawBody;
    const key = await getPublicKey(publicKey);
    // @ts-ignore ????
    const isVerified = await webcrypto.subtle.verify("NODE-ED25519", key, signature, encoder.encode(timestamp + body));
    if (signature == null || timestamp == null || body == null || !isVerified) {
        return false;
    }
    return true;
}
/* eslint-enable */
//# sourceMappingURL=validate.js.map