import fastify from "fastify";
import rawBody from "fastify-raw-body";
import { validateRequest } from "./requests/validate";
export default async function startInteractionKitServer(handler, publicKey, port = 3000) {
    const server = fastify();
    void server.register(rawBody, {
        runFirst: true,
    });
    server.addHook("preHandler", async (request, response) => {
        if (request.method === "POST") {
            const isValid = await validateRequest(request, publicKey);
            if (!isValid) {
                console.log("Invalid Discord Request");
                return response.status(401).send({ error: "Bad request signature" });
            }
        }
    });
    server.post("/", async (request, response) => {
        handler(request, response);
    });
    server.listen(port, async (error, address) => {
        if (error) {
            console.error(error);
            process.exit(1);
        }
        console.log(`Server listening on ${address}`);
    });
    return new Promise((resolve) => {
        server.ready(() => {
            resolve(server);
        });
    });
}
//# sourceMappingURL=server.js.map